@@ -33,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.9 2003/06/11 11:42:31 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.10 2005/08/08 19:46:38 abe Exp $";
 #endif
 
 
@@ -277,14 +277,14 @@ readafsnode(va, v, an)
  */
 	if (vc->mvstat == 2) {
 	    if ((vp = getvolume(&vc->fid, &vols))) {
-		an->inode = (unsigned long)((vp->mtpoint.Fid.Vnode
-			  +		     (vp->mtpoint.Fid.Volume << 16))
+		an->inode = (INODETYPE)((vp->mtpoint.Fid.Vnode
+			  +		(vp->mtpoint.Fid.Volume << 16))
 			  & 0x7fffffff);
-		if (an->inode == (unsigned long)0) {
+		if (an->inode == (INODETYPE)0) {
 		    if (is_rootFid(vc, &rfid))
 			an->ino_st = 1;
 		    else if (rfid) {
-			an->inode = (unsigned long)2;
+			an->inode = (INODETYPE)2;
 			an->ino_st = 1;
 		    } else
 			an->ino_st = 0;
@@ -292,19 +292,19 @@ readafsnode(va, v, an)
 		    an->ino_st = 1;
 	    } else {
 		if (vols) {
-		    an->inode = (unsigned long)2;
+		    an->inode = (INODETYPE)2;
 		    an->ino_st = 1;
 		} else {
 		    if (v->v_flag & V_ROOT) {
-			an->inode = (unsigned long)0;
+			an->inode = (INODETYPE)0;
 			an->ino_st = 1;
 		    } else
 			an->ino_st = 0;
 		}
 	    }
 	} else {
-	    an->inode = (unsigned long)((vc->fid.Fid.Vnode
-		      +			(vc->fid.Fid.Volume << 16))
+	    an->inode = (INODETYPE)((vc->fid.Fid.Vnode
+		      +		    (vc->fid.Fid.Volume << 16))
 		      & 0x7fffffff);
 	    an->ino_st = 1;
 	}
