@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.23 2005/08/08 19:46:38 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.24 2008/10/21 16:14:18 abe Exp $";
 #endif
 
 
@@ -70,6 +70,8 @@ process_socket(sa)
 	struct protosw p;
 	struct socket s;
 	struct tcpcb t;
+	int ts = 0;
+	int tsn, tsnx;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = (struct sockaddr_un *)NULL;
 	struct sockaddr_un un;
@@ -160,7 +162,63 @@ process_socket(sa)
 	case AF_INET6:
 #endif	/* defined(HASIPv6) */
 
+	/*
+	 * Read protocol control block.
+	 */
+	    if (!s.so_pcb
+	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
+		if (!s.so_pcb) {
+		    (void) snpf(Namech, Namechl, "no PCB%s%s",
+			(s.so_state & SS_CANTSENDMORE) ? ", CANTSENDMORE" : "",
+			(s.so_state & SS_CANTRCVMORE)  ? ", CANTRCVMORE"  : "");
+		} else {
+		    (void) snpf(Namech, Namechl, "can't read inpcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
+		}
+		enter_nm(Namech);
+		return;
+	    }
+	    if (p.pr_protocol == IPPROTO_TCP) {
+
+	    /*
+	     * If this is a TCP socket, read its control block.
+	     */
+		if (inp.inp_ppcb
+	        &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t)))
+		{
+		    ts = 1;
+		    tsn = (int)t.t_state;
+		    tsnx = tsn + TcpStOff;
+		}
+	    }
+	    if (ts
+	    &&  (TcpStIn || TcpStXn)
+	    &&  (tsnx >= 0) && (tsnx < TcpNstates)
+	    ) {
+
+	    /*
+	     * Check TCP state name inclusion and exclusions.
+	     */
+		if (TcpStXn) {
+		    if (TcpStX[tsnx]) {
+			Lf->sf |= SELEXCLF;
+			return;
+		    }
+		}
+		if (TcpStIn) {
+		    if (TcpStI[tsnx])
+			TcpStI[tsnx] = 2;
+		    else {
+			Lf->sf |= SELEXCLF;
+			return;
+		    }
+		}
+	    }
 	    if (Fnet) {
+
+	    /*
+	     * Set SELNET flag for the file, as requested.
+	     */
 		if (!FnetTy
 		||  ((FnetTy == 4) && (fam == AF_INET))
 
@@ -180,22 +238,6 @@ process_socket(sa)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
 #endif	/* defined(HASIPv6) */
 
-	/*
-	 * Read protocol control block.
-	 */
-	    if (!s.so_pcb
-	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
-		if (!s.so_pcb) {
-		    (void) snpf(Namech, Namechl, "no PCB%s%s",
-			(s.so_state & SS_CANTSENDMORE) ? ", CANTSENDMORE" : "",
-			(s.so_state & SS_CANTRCVMORE)  ? ", CANTRCVMORE"  : "");
-		} else {
-		    (void) snpf(Namech, Namechl, "can't read inpcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
-		}
-		enter_nm(Namech);
-		return;
-	    }
 	/*
 	 * Save Internet socket information.
 	 */
@@ -255,10 +297,9 @@ process_socket(sa)
 
 	    if (fa || la)
 		(void) ent_inaddr(la, lp, fa, fp, fam);
-	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
-	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+	    if (ts) {
 		Lf->lts.type = 0;
-		Lf->lts.state.i = (int)t.t_state;
+		Lf->lts.state.i = tsn;
 
 #if	defined(HASSOOPT)
 		Lf->lts.kai = (unsigned int)t.t_timer[TCPT_KEEP];
