@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.46 2014/10/13 22:25:07 abe Exp $
+ * $Id: dlsof.h,v 1.48 2018/02/14 14:26:03 abe Exp $
  */
 
 
@@ -87,6 +87,14 @@
 #define	boolean_t	int
 #  endif	/* defined(NEEDS_BOOLEAN_T) */
 
+/*
+ * Define KLD_MODULE to avoid the error "ARM_NMMUS is 0" from ARM's
+ * <machine/cpuconf.h>.
+ */
+
+#define KLD_MODULE
+
+
 #include <sys/conf.h>
 
 #  if	defined(HAS_VM_MEMATTR_T)
@@ -134,8 +142,10 @@ int     getmntinfo(struct statfs **, int);
 #include <rpc/types.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
+#define	_WANT_SOCKET
 #include <sys/socketvar.h>
 #include <sys/un.h>
+#define	_WANT_UNPCB
 #include <sys/unpcb.h>
 
 # if	FREEBSDV>=3000
@@ -146,12 +156,14 @@ int     getmntinfo(struct statfs **, int);
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <net/route.h>
+#define	_WANT_INPCB			/* for FreeBSD 12 and above */
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
 #include <netinet/tcp.h>
 #include <netinet/tcpip.h>
 #include <netinet/tcp_fsm.h>
 #include <netinet/tcp_timer.h>
+#define	_WANT_TCPCB			/* for FreeBSD 12 and above */
 #include <netinet/tcp_var.h>
 #include <sys/ucred.h>
 #include <sys/uio.h>
@@ -340,6 +352,8 @@ struct vop_generic_args;
 #  endif	/* FREEBSDV<5000 */
 # endif	/* FREEBSDV<2000 */
 
+#include <vm/vm.h>
+
 #define	_KERNEL
 #define	KERNEL
 #include <sys/fcntl.h>
@@ -370,7 +384,9 @@ int     open(const char *, int, ...);
 #define	pause	pause_kernel_lsof
 # endif	/* !defined(HAS_PAUSE_SBT) */
 
+#define	asprintf asprintf_kernel_lsof
 #define	setenv	setenv_kernel_lsof
+#define	vasprintf vasprintf_kernel_lsof
 #define	uintfptr_t	int
 #define	_SYS_LIBKERN_H_
 #include <sys/file.h>
@@ -380,6 +396,7 @@ int     open(const char *, int, ...);
  */
 
 #undef	_SYS_LIBKERN_H_
+#undef	asprintf_kernel_lsof
 #undef	intrmask_t_lsof
 #undef	log_kernel_lsof
 
@@ -388,6 +405,7 @@ int     open(const char *, int, ...);
 # endif	/* !defined(HAS_PAUSE_SBT) */
 
 #undef	setenv_kernel_lsof
+#undef	vasprintf_kernel_lsof
 #undef	uintfptr_t
 #undef	_KERNEL
 #undef	KERNEL
@@ -427,8 +445,6 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #include <sys/lockf.h>
 # endif	/* FREEBSDV<2000 */
 
-#include <vm/vm.h>
-
 #  if   FREEBSDV>=2020
 #   if	FREEBSDV>=4090
 #define	_KERNEL
@@ -472,6 +488,7 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #  endif	/* FREEBSDV<5000 */
 # endif        /* FREEBSDV>=2020 */
 
+#undef	bzero		/* avoid _KERNEL conflict */
 #include <string.h>
 
 
@@ -557,6 +574,8 @@ struct mounts {
         struct mounts *next;    	/* forward link */
 };
 
+#define	X_BADFILEOPS	"badfileops"
+extern KA_T X_bfopsa;
 #define	X_NCACHE	"ncache"
 #define	X_NCSIZE	"ncsize"
 #define	NL_NAME		n_name
