@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.18 2014/10/13 22:25:07 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.20 2018/02/14 14:26:03 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -127,6 +127,11 @@ gather_proc_info()
 	typedef struct file* ofb_t;
 #endif	/* defined(HAS_FILEDESCENT) */
 
+#if	defined(HAS_FDESCENTTBL)
+	struct fdescenttbl fdt;
+	KA_T fa;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
 	static ofb_t *ofb = NULL;
 	static int ofbb = 0;
 	int pgid, pid;
@@ -185,7 +190,7 @@ gather_proc_info()
 	     * If only ORed process selection options have been specified,
 	     * enable conditional file skipping and socket file only checking.
 	     */
-		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		if ((Selflags & SELFILE) || !(Selflags & SelProc))
 		    cckreg = ckscko = 0;
 		else
 		    cckreg = ckscko = 1;
@@ -295,8 +300,18 @@ gather_proc_info()
 	    if (!p->P_FD
 	    ||  kread((KA_T)p->P_FD, (char *)&fd, sizeof(fd)))
 		continue;
+
+#if	defined(HAS_FDESCENTTBL)
+	    if (!fd.fd_files
+	    ||  kread((KA_T)fd.fd_files, (char *)&fdt, sizeof(fdt)))
+		continue;
+	    if (!fd.fd_refcnt || fd.fd_lastfile > fdt.fdt_nfiles)
+		continue;
+#else	/* !defined(HAS_FDESCENTTBL) */
 	    if (!fd.fd_refcnt || fd.fd_lastfile > fd.fd_nfiles)
 		continue;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
 	/*
 	 * Allocate a local process structure.
 	 */
@@ -309,7 +324,7 @@ gather_proc_info()
 	     * socket file only checking, based on the process' selection
 	     * status.
 	     */
-		ckscko = (sf & SELPROC) ? 0 : 1;
+		ckscko = (sf & SelProc) ? 0 : 1;
 	    }
 	    alloc_lproc(p->P_PID, pgid, ppid, (UID_ARG)uid, p->P_COMM,
 		(int)pss, (int)sf);
@@ -371,8 +386,15 @@ gather_proc_info()
 	/*
 	 * Read open file structure pointers.
 	 */
+
+#if	defined(HAS_FDESCENTTBL)
+	    if ((nf = fdt.fdt_nfiles) <= 0)
+		continue;
+#else	/* !defined(HAS_FDESCENTTBL) */
 	    if (!fd.fd_ofiles || (nf = fd.fd_nfiles) <= 0)
 		continue;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
 	    nb = (MALLOC_S)(sizeof(ofb_t) * nf);
 	    if (nb > ofbb) {
 		if (!ofb)
@@ -386,8 +408,17 @@ gather_proc_info()
 		}
 		ofbb = nb;
 	    }
+
+#if	defined(HAS_FDESCENTTBL)
+	    fa = (KA_T)fd.fd_files
+	       + (KA_T)offsetof(struct fdescenttbl, fdt_ofiles);
+	    if (kread(fa, (char *)ofb, nb))
+		continue;
+#else	/* !defined(HAS_FDESCENTTBL) */
 	    if (kread((KA_T)fd.fd_ofiles, (char *)ofb, nb))
 		continue;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
 
 #if	defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT)
 	    if (Fsv & FSV_FG) {
@@ -534,6 +565,17 @@ get_kernel_access()
 	    Exit(1);
 	}
 
+#if	defined(X_BADFILEOPS)
+/*
+ * Get kernel's badfileops address (for process_file()).
+ */
+	if (get_Nl_value(X_BADFILEOPS, (struct drive_Nl *)NULL, &X_bfopsa) < 0
+	||  !X_bfopsa)
+	{
+	    X_bfopsa = (KA_T)0;
+	}
+#endif	/* defined(X_BADFILEOPS) */
+
 #if	defined(WILLDROPGID)
 /*
  * Drop setgid permission, if necessary.
