@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.37 2008/10/21 16:16:06 abe Exp $
+ * $Id: machine.h,v 1.40 2018/02/14 14:26:03 abe Exp $
  */
 
 
@@ -62,6 +62,20 @@
 typedef	__cpumask_t	cpumask_t;
 #endif	/* defined(HASCPUMASK_T) */
 
+#if	defined(NEEDS_BOOL_TYPEDEF)
+/*
+ * Under FreeBSD 10.0 on the i386 architecture the bool typedef is not defined
+ * unless <sys/types.h> is #included under _KERNEL.  As explained with
+ * cpumask_t, #include'ing <sys/types.h> under _KERNEL causes other problems.
+ * Hence Configure specifies that bool be typedef'd here specifically for the
+ * i386 architecture on FreeBSD 10.0.
+ */
+# if	__STDC_VERSION__<199901L && __GNUC__<3 && !defined(__INTEL_COMPILER)
+typedef int     _Bool;
+# endif	/*__STDC_VERSION__<199901L && __GNUC__<3 && !defined(__INTEL_COMPILER)*/
+typedef _Bool   bool;
+#endif	/* defined(NEEDS_BOOL_TYPEDEF) */
+
 #include <sys/param.h>
 
 
@@ -426,6 +440,17 @@ typedef	__cpumask_t	cpumask_t;
 # endif	/* FREEBSDV>=2000 */
 
 
+/*
+ * HASPTSFN is defined for dialects that have a DTYPE_PTS file descriptor
+ * type.  The definition names the function that processes the type.
+ *
+ *
+ * NOTE: don't forget to define a prototype for this function in dproto.h.
+ */
+
+#define	HASPTSFN	process_pts
+
+
 /*
  * HASRNODE is defined for those dialects that have rnodes.
  */
@@ -478,6 +503,13 @@ typedef	__cpumask_t	cpumask_t;
 /* #define	HASSNODE	1 */
 
 
+/*
+ * HASTASKS is defined for those dialects that have task reporting support.
+ */
+
+/* #define	HASTASKS	1 */
+
+
 /*
  * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
  * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
