@@ -32,17 +32,28 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.33 2004/07/06 19:20:18 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.38 2007/04/24 16:22:02 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+#if	defined(HAS_DINODE_U)
+#define	DINODE_U	dinode_u
+#else	/* !defined(HAS_DINODE_U) */
+#define	DINODE_U	i_din
+#endif	/* defined(HAS_DINODE_U) */
+
 #if	defined(HASFDESCFS) && HASFDESCFS==1
-_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
+_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, INODETYPE *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
+#if	defined(HAS_UM_UFS)
+#define	UFS1	UM_UFS1
+#define	UFS2	UM_UFS2
+#endif	/* defined(HAS_UM_UFS) */
+
 #if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
 
@@ -98,7 +109,7 @@ getmemsz(pid)
 static int
 lkup_dev_tty(dr, ir)
 	dev_t *dr;			/* place to return device number */
-	unsigned long *ir;		/* place to return inode number */
+	INODETYPE *ir;			/* place to return inode number */
 {
 	int i;
 
@@ -119,7 +130,7 @@ lkup_dev_tty_again:
 # endif	/* defined(HASDCACHE) */
 
 		*dr = Devtp[i].rdev;
-		*ir = (unsigned long)Devtp[i].inode;
+		*ir = Devtp[i].inode;
 		return(1);
 	    }
 	}
@@ -188,16 +199,17 @@ process_node(va)
 	struct mfsnode m;
 	struct nfsnode n;
 	enum nodetype {NONODE, CDFSNODE, DOSNODE, EXT2NODE, FDESCNODE, INODE,
-		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE} nty;
+		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE, PTYFSNODE} nty;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
 
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
-	unsigned long iso_ino, iso_sz;
+	INODETYPE iso_ino;
 	long iso_nlink;
 	int iso_stat;
+	SZOFFTYPE iso_sz;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASFDESCFS)
@@ -205,7 +217,7 @@ process_node(va)
 
 # if	HASFDESCFS==1
 	static dev_t f_tty_dev;
-	static unsigned long f_tty_ino;
+	static INODETYPE f_tty_ino;
 	static int f_tty_s = 0;
 # endif	/* HASFDESCFS==1 */
 
@@ -239,7 +251,7 @@ process_node(va)
 #if	defined(HASMSDOSFS)
 	struct denode d;
 	u_long dpb;
-	u_long nn;
+	INODETYPE nn;
 	struct msdosfsmount pm;
 #endif	/* defined(HASMSDOSFS) */
 
@@ -262,6 +274,11 @@ process_node(va)
 	size_t sz;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	struct ptyfsnode pt;
+	struct specinfo si;
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASNULLFS)
 
 process_overlaid_node:
@@ -364,8 +381,7 @@ process_overlaid_node:
 
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
-	    if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
-		  &iso_nlink, (SZOFFTYPE *)&iso_sz))
+	    if (read_iso_node(v, &iso_dev, &iso_ino, &iso_nlink, &iso_sz))
 	    {
 		(void) snpf(Namech, Namechl, "can't read iso_node at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
@@ -435,7 +451,7 @@ process_overlaid_node:
 	 */
 	    if (v->v_type == VDIR && (v->v_flag & VROOT)) {
 		(void) snpf(Namech, Namechl, "%s", _PATH_KERNFS);
-		ksb.st_ino = 2;
+		ksb.st_ino = (ino_t)2;
 		ksb.st_size = DEV_BSIZE;
 		ksbs = 1;
 	    } else if (Namech[0] && statsafely(Namech, &ksb) == 0)
@@ -523,6 +539,19 @@ process_overlaid_node:
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if    defined(HASPTYFS)
+	case VT_PTYFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&pt, sizeof(pt))) {
+		(void) snpf(Namech, Namechl, "can't read ptyfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = PTYFSNODE;
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASEXT2FS)
 	case VT_EXT2FS:
 #endif	/* defined(HASEXT2FS) */
@@ -545,12 +574,12 @@ process_overlaid_node:
 		nty = EXT2NODE;
 
 # if	defined(HASI_E2FS_PTR)
-		if (i.i_din.e2fs_din
-		&&  !kread((KA_T)i.i_din.e2fs_din, (char *)&ed, sizeof(ed)))
+		if (i.DINODE_U.e2fs_din
+		&&  !kread((KA_T)i.DINODE_U.e2fs_din, (char *)&ed, sizeof(ed)))
 		    edp = &ed;
 # else	/* !defined(HASI_E2FS_PTR) */
 #  if	HASEXT2FS<2
-		edp = &i.i_din.e2fs_din;
+		edp = &i.DINODE_U.e2fs_din;
 #  else	/* HASEXT2FS>=2 */
 		edp = &i.i_e2din;
 #  endif	/* HASEXT2FS>=2 */
@@ -570,16 +599,16 @@ process_overlaid_node:
 		    &&  !kread((KA_T)i.i_ump, (char *)&um, sizeof(um))) {
 			if (um.um_fstype == UFS1) {
 			    ffs = 1;
-			    if (i.i_din.ffs1_din
-			    &&  !kread((KA_T)i.i_din.ffs1_din, (char *)&u1,
+			    if (i.DINODE_U.ffs1_din
+			    &&  !kread((KA_T)i.DINODE_U.ffs1_din, (char *)&u1,
 				       sizeof(u1)))
 			    {
 				u1s = 1;
 			    }
 			} else if (um.um_fstype == UFS2) {
 			    ffs = 2;
-			    if (i.i_din.ffs2_din
-			    &&  !kread((KA_T)i.i_din.ffs2_din, (char *)&u2,
+			    if (i.DINODE_U.ffs2_din
+			    &&  !kread((KA_T)i.DINODE_U.ffs2_din, (char *)&u2,
 				       sizeof(u2)))
 			    {
 				u2s = 1;
@@ -701,7 +730,7 @@ process_overlaid_node:
 		}
 # else	/* !defined(HASI_E2FS_PTR) */
 #  if	HASEXT2FS<2
-		rdev = i.i_din.e2fs_din.e2di_rdev;
+		rdev = i.DINODE_U.e2fs_din.e2di_rdev;
 #  else	/* HASEXT2FS>=2 */
 		rdev = i.i_e2din.e2di_rdev;
 #  endif	/* HASEXT2FS>=2 */
@@ -769,6 +798,28 @@ process_overlaid_node:
 	case NFSNODE:
 	    dev = NVATTR.va_fsid;
 	    devs = 1;
+	    break;
+
+#if	defined(HASPTYFS)
+	case PTYFSNODE:
+	    if (v->v_un.vu_specinfo
+	    &&  !kread((KA_T)v->v_un.vu_specinfo, (char *)&si, sizeof(si))) {
+		rdev = si.si_rdev;
+		rdevs = 1;
+	    }
+	    if (vfs) {
+
+# if	defined(HASSTATVFS)
+		dev = (dev_t)vfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
+		dev = (dev_t)vfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
+		devs = 1;
+	    }
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 	}
 /*
  * Obtain the inode number.
@@ -781,17 +832,17 @@ process_overlaid_node:
 		dpb = (u_long)(pm.pm_BytesPerSec / sizeof(struct direntry));
 		if (d.de_Attributes & ATTR_DIRECTORY) {
 		    if (d.de_StartCluster == MSDOSFSROOT)
-			nn = (u_long)1;
+			nn = (INODETYPE)1;
 		    else
-			nn = (u_long)(cntobn(&pm, d.de_StartCluster) * dpb);
+			nn = (INODETYPE)(cntobn(&pm, d.de_StartCluster) * dpb);
 		} else {
 		    if (d.de_dirclust == MSDOSFSROOT)
-			nn = (u_long)(roottobn(&pm, 0) * dpb);
+			nn = (INODETYPE)(roottobn(&pm, 0) * dpb);
 		    else
-			nn = (u_long)(cntobn(&pm, d.de_dirclust) * dpb);
-		    nn += (u_long)(d.de_diroffset / sizeof(struct direntry));
+			nn = (INODETYPE)(cntobn(&pm, d.de_dirclust) * dpb);
+		    nn += (INODETYPE)(d.de_diroffset / sizeof(struct direntry));
 		}
-		Lf->inode = (unsigned long)nn;
+		Lf->inode = nn;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -802,14 +853,14 @@ process_overlaid_node:
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-	    Lf->inode = (unsigned long)i.i_number;
+	    Lf->inode = (INODETYPE)i.i_number;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
 	    if (ksbs) {
-		Lf->inode = (unsigned long)ksb.st_ino;
+		Lf->inode = (INODETYPE)ksb.st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -825,17 +876,30 @@ process_overlaid_node:
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-	    Lf->inode = (unsigned long)NVATTR.va_fileid;
+	    Lf->inode = (INODETYPE)NVATTR.va_fileid;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASPROCFS)
 	case PFSNODE:
-	    Lf->inode = (unsigned long)p.pfs_fileno;
+	    Lf->inode = (INODETYPE)p.pfs_fileno;
 	    Lf->inp_ty = 1;
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	case PTYFSNODE:
+	    if (pt.ptyfs_type == PTYFSptc) {
+		if (pt.ptyfs_fileno > 0x3fffffff)
+		    Lf->inode = (INODETYPE)(pt.ptyfs_fileno & 0x3fffffff);
+		else
+		    Lf->inode = (INODETYPE)(pt.ptyfs_fileno - 1);
+	    } else
+		Lf->inode = (INODETYPE)pt.ptyfs_fileno;
+	    Lf->inp_ty = 1;
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 	}
 
 /*
@@ -1009,27 +1073,26 @@ process_overlaid_node:
 		switch (nty) {
 		case INODE:
 
+#if	defined(HASEFFNLINK)
+		    Lf->nlink = (long)i.HASEFFNLINK;
+#else	/* !defined(HASEFFNLINK) */
 # if	defined(HASI_FFS)
 		    Lf->nlink = (long)i.i_ffs_nlink;
-		    Lf->nlink_def = 1;
 # else	/* !defined(HASI_FFS) */
 #  if	defined(HASI_FFS1)
 		    if (ffs == 1) {
-			if (u1s) {
+			if (u1s)
 			    Lf->nlink = (long)u1.di_nlink;
-			    Lf->nlink_def = 1;
-			}
 		    } else if (ffs == 2) {
-			if (u2s) {
+			if (u2s)
 			    Lf->nlink = (long)u2.di_nlink;
-			    Lf->nlink_def = 1;
-			}
 		    }
 #  else	/* !defined(HASI_FFS1) */
+
 		    Lf->nlink = (long)i.i_nlink;
-		    Lf->nlink_def = 1;
 #  endif	/* defined(HASI_FFS1) */
 # endif	/* defined(HASI_FFS) */
+#endif	/* defined(HASEFFNLINK) */
 
 		    Lf->nlink_def = 1;
 		    break;
@@ -1136,11 +1199,7 @@ process_overlaid_node:
 	    ty = "FIFO";
 	    break;
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = NULL;
 	}
 	if (ty)
@@ -1152,7 +1211,8 @@ process_overlaid_node:
  * 	ioctl(fd, TIOCNOTTY) files;
  *	/kern files
  *	memory node files;
- *	/proc files.
+ *	/proc files;
+ *	ptyfs files.
  */
 
 	if (type == VBAD)
@@ -1222,14 +1282,62 @@ process_overlaid_node:
 		(void) snpf(ep, sz, "/%d/notepg", p.pfs_pid);
 		ty = "PGID";
 		break;
+
+# if	defined(Pfd)
+	    case Pfd:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/fd", p.pfs_pid);
+		ty = "PFD";
+		break;
+# endif	/* defined(Pfd) */
+
+# if	defined(Pmap)
+	    case Pmap:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/map", p.pfs_pid);
+		ty = "PMAP";
+		break;
+# endif	/* defined(Pmap) */
+
+# if	defined(Pmaps)
+	    case Pmaps:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/maps", p.pfs_pid);
+		ty = "PMPS";
+		break;
+# endif	/* defined(Pmaps) */
+
 	    }
 	    if (ty)
 		(void) snpf(Lf->type, sizeof(Lf->type), ty);
-	    if (Namech[0])
-		enter_nm(Namech);
 	}
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	else if (nty == PTYFSNODE) {
+	    (void) snpf(Namech, Namechl, "%s", Lf->fsdir);
+	    Lf->nlink = 1;
+	    Lf->nlink_def = 1;
+	    switch (pt.ptyfs_type) {
+	    case PTYFSpts:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%lu", (unsigned long)pt.ptyfs_pty);
+		break;
+	    case PTYFSptc:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%lu (master)",
+		    (unsigned long)pt.ptyfs_pty);
+		break;
+	    case PTYFSroot:
+		Lf->sz = 512;
+		Lf->sz_def = 1;
+		break;
+	    }
+	    if (ty)
+		(void) snpf(Lf->type, sizeof(Lf->type), ty);
+	}
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASBLKDEV)
 /*
  * If this is a VBLK file and it's missing an inode number, try to
@@ -1259,7 +1367,7 @@ process_overlaid_node:
 		    if ((pfi->pid && pfi->pid == p.pfs_pid)
 		    
 # if	defined(HASPINODEN)
-		    ||  (Lf->inp_ty == 1 && pfi->inode == Lf->inode)
+		    ||  ((Lf->inp_ty == 1) && (pfi->inode == Lf->inode))
 # endif	/* defined(HASPINODEN) */
 
 		    ) {
@@ -1283,7 +1391,9 @@ process_overlaid_node:
 	    if (Sfile && is_file_named((char *)NULL,
 				       ((type == VCHR) || (type == VBLK)) ? 1
 									  : 0))
+	    {
 		Lf->sf |= SELNM;
+	    }
 	    if (ns)
 		Namech[0] = '\0';
 	}
