@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.23 2004/03/10 23:51:34 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.25 2005/08/08 19:53:24 abe Exp $";
 #endif
 
 
@@ -79,9 +79,9 @@ process_socket(sa)
 #define	UNPADDR_IN_MBUF
 
 #if	defined(NETBSDV)
-# if	NETBSDV>=1040
+# if	NETBSDV>=1004000
 #undef	UNPADDR_IN_MBUF
-# endif	/* NETBSDV>=1040 */
+# endif	/* NETBSDV>=1004000 */
 #endif	/* defined(NETBSDV) */
 
 #if	defined(UNPADDR_IN_MBUF)
@@ -144,12 +144,13 @@ process_socket(sa)
 #if	defined(HASSOOPT)
 	Lf->lts.ltm = (unsigned int)s.so_linger;
 	Lf->lts.opt = (unsigned int)s.so_options;
+	Lf->lts.pqlen = (unsigned int)s.so_q0len;
 	Lf->lts.qlen = (unsigned int)s.so_qlen;
 	Lf->lts.qlim = (unsigned int)s.so_qlimit;
 	Lf->lts.rbsz = (unsigned long)s.so_rcv.sb_mbmax;
 	Lf->lts.sbsz = (unsigned long)s.so_snd.sb_mbmax;
-	Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
-		      = Lf->lts.sbszs = (unsigned char)1;
+	Lf->lts.pqlens = Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+		       = Lf->lts.sbszs = (unsigned char)1;
 #endif	/* defined(HASSOOPT) */
 
 #if	defined(HASSOSTATE)
