@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.20 2005/08/08 19:54:32 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.21 2006/03/28 22:09:23 abe Exp $";
 #endif
 
 
@@ -667,11 +667,8 @@ process_node(na)
 #endif	/* defined(IFLNK) */
 
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown inode type: %x", type);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o",
+		((type >> 12) & 0xfff));
 	    tn = NULL;
 	}
 	if (tn)
