@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.27 2013/01/02 17:14:59 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.29 2018/02/14 14:20:14 abe Exp $";
 #endif
 
 
@@ -73,8 +73,6 @@ _PROTOTYPE(static int handleint,(int sig));
 _PROTOTYPE(static void handleint,(int sig));
 #endif	/* defined(HASINTSIGNAL) */
 
-_PROTOTYPE(static char *safepup,(unsigned int c, int *cl));
-
 
 /*
  * Local variables
@@ -249,6 +247,7 @@ doinchild(fn, fp, rbuf, rbln)
 	int rbln;			/* response buffer length */
 {
 	int en, rv;
+
 /*
  * Check reply buffer size.
  */
@@ -294,14 +293,41 @@ doinchild(fn, fp, rbuf, rbln)
 		 * Begin the child process.
 		 */
 
-		    int fd, nd, r_al, r_rbln;
+		    int r_al, r_rbln;
 		    char r_arg[MAXPATHLEN+1], r_rbuf[MAXPATHLEN+1];
 		    int (*r_fn)();
 		/*
-		 * Close all open file descriptors except Pipes[0] and
+		 * Close sufficient open file descriptors except Pipes[0] and
 		 * Pipes[3].
 		 */
-		    for (fd = 0, nd = GET_MAX_FD(); fd < nd; fd++) {
+
+#if	defined(HAS_DUP2) && defined(HAS_CLOSEFROM)
+		    int rc;
+
+		    rc = dup2(Pipes[0], 0);
+		    if (rc < 0) {
+			(void) fprintf(stderr,
+			    "%s: can't dup Pipes[0] to fd 0: %s\n",
+			    Pn, strerror(errno));
+			Exit(1);
+		    }
+		    Pipes[0] = 0;
+		    rc = dup2(Pipes[3], 1);
+		    if (rc < 0) {
+			(void) fprintf(stderr,
+			    "%s: can't dup Pipes.[3] to fd 1: %s\n",
+			    Pn, strerror(errno));
+			Exit(1);
+		    }
+		    Pipes[3] = 1;
+		    (void) closefrom(2);
+		    Pipes[1] = -1;
+		    Pipes[2] = -1;
+
+#else	/* !defined(HAS_DUP2) && !defined(HAS_CLOSEFROM) */
+		    int fd;
+
+		    for (fd = 0; fd < MaxFd; fd++) {
 			if (fd == Pipes[0] || fd == Pipes[3])
 			    continue;
 			(void) close(fd);
@@ -318,6 +344,8 @@ doinchild(fn, fp, rbuf, rbln)
 			(void) close(Pipes[2]);
 			Pipes[2] = -1;
 		    }
+#endif	/* defined(HAS_DUP2) && defined(HAS_CLOSEFROM) */
+
 		/*
 		 * Read function requests, process them, and return replies.
 		 */
@@ -1314,10 +1342,10 @@ readstqinit(addr, buf)
  *	   cl = strlen(printable equivalent)
  */
 
-static char *
+char *
 safepup(c, cl)
 	unsigned int c;			/* unprintable (i.e., !isprint())
-					 * character */
+					 * character  and '\\' */
 	int *cl;			/* returned printable strlen -- NULL if
 					 * no return needed */
 {
@@ -1350,6 +1378,9 @@ safepup(c, cl)
 	} else if (c == 0xff) {
 	    rp = "^?";
 	    len = 2;
+	} else if (c == '\\') {
+	    rp = "\\\\";
+	    len = 2;
 	} else {
 	    (void) snpf(up, sizeof(up), "\\x%02x", (int)(c & 0xff));
 	    rp = up;
@@ -1382,8 +1413,11 @@ safestrlen(sp, flags)
 	c = (flags & 2) ? ' ' : '\0';
 	if (sp) {
 	    for (; *sp; sp++) {
-		if (!isprint((unsigned char)*sp) || *sp == c) {
-		    if (*sp < 0x20 || (unsigned char)*sp == 0xff)
+		if (!isprint((unsigned char)*sp)
+		||  (*sp == '\\') || (*sp == c))
+		{
+		    if ((*sp < 0x20) || ((unsigned char)*sp == 0xff)
+		    ||  (*sp == '\\'))
 			len += 2;		/* length of \. or ^. form */
 		    else
 			len += 4;		/* length of "\x%02x" printf */
@@ -1457,7 +1491,7 @@ safestrprt(sp, fs, flags)
 		lnc = 1;
 #endif	/* defined(HASWIDECHAR) */
 
-		if (isprint((unsigned char)*sp) && *sp != c)
+		if ((*sp != '\\') && isprint((unsigned char)*sp) && *sp != c)
 		    putc((int)(*sp & 0xff), fs);
 		else {
 		    if ((flags & 8) && (*sp == '\n') && !*(sp + 1))
@@ -1506,7 +1540,7 @@ safestrprtn(sp, len, fs, flags)
 	if (sp) {
 	    c = (flags & 2) ? ' ' : '\0';
 	    for (i = 0; i < len && *sp; sp++) {
-		if (isprint((unsigned char)*sp) && *sp != c) {
+		if ((*sp != '\\') && isprint((unsigned char)*sp) && *sp != c) {
 		    putc((int)(*sp & 0xff), fs);
 		    i++;
 		} else {
