@@ -99,6 +99,21 @@ static char copyright[] =
 #endif	/* defined(LT_DIAL_bsdi) */
 
 
+#if	defined(LT_DIAL_freebsd)
+/*
+ *FreeBSD-specific items
+*/
+
+#undef	XDINDEV
+#define	XDINDEV		16
+# if	defined(LT_DEV64)
+#undef	X2DEV_T
+#define	X2DEV_T		unsigned long long
+#define	major_X(dp, em)	((int)((x2dev(dp, em) >> 32) & 0xffffffff))
+# endif	/* defined(LT_DEV64) */
+#endif	/* defined(LT_DIAL_freebsd) */
+
+
 #if	defined(LT_DIAL_osr)
 /*
  * OpenUNIX-specific items
@@ -537,7 +552,12 @@ ExecLsof(opt)
 	if (!(LsofFs = fdopen(LsofFd, "r")))
 	    return("ERROR!!!  ExecLsof() can't open stream to lsof output FD");
     }
-    return((char *)NULL);			/* All is well. */
+/*
+ * Wait a bit for lsof to start and put something in its pipe, then return
+ * an "All is well." response.
+ */
+    sleep(1);
+    return((char *)NULL);
 }
 
 
@@ -918,7 +938,7 @@ RdFrLsof(nf, em)
 	if (!(vp = (char *)malloc(tlen + 1))) {
 
 	/*
-	 * A serios error has occurred; there's no space for the field value.
+	 * A serious error has occurred; there's no space for the field value.
 	 */
 	    (void) snprintf(buf, bufl,
 		"ERROR!!!  RdFrLsof() can't allocate %d field bytes", tlen + 1);
